package com.example.wholeestate.controller;


import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.example.wholeestate.common.base.BaseController;
import com.example.wholeestate.common.resp.ResponseBean;
import com.example.wholeestate.common.uuid.IdGen;
import com.example.wholeestate.model.Activity;
import com.example.wholeestate.service.IActivityEnrollService;
import com.example.wholeestate.service.IActivityService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.Date;

/**
 * <p>
 * 活动表 前端控制器
 * </p>
 *
 * @author weiximei
 * @since 2019-03-20
 */
@RestController
@RequestMapping("/")
@Api(value = "活动 API", description = "活动 API")
public class ActivityController extends BaseController {

    private IdGen idGen = IdGen.get();

    @Autowired
    private IActivityService iActivityService;

    @Autowired
    private IActivityEnrollService iActivityEnrollService;

    /**
     * 自定义方法绑定请求参数的Date类型
     *
     * @param request
     * @param binder
     * @throws Exception
     */
    @InitBinder
    protected void initBinder(HttpServletRequest request, ServletRequestDataBinder binder) throws Exception {
        DateFormat df = new SimpleDateFormat("yyyy-MM-dd");
        CustomDateEditor editor = new CustomDateEditor(df, true);//true表示允许为空，false反之
        binder.registerCustomEditor(Date.class, editor);
    }


    @RequestMapping(value = "/activity/list", method = RequestMethod.GET)
    @ApiOperation(value = "活动列表", notes = "活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageNum", value = "pageNum第几页"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageSize", value = "pageSize一页多少行"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = String.class, name = "name", value = "活动名称"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Date.class, name = "beginDate", value = "开始时间"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Date.class, name = "endDate", value = "结束时间")
    })
    public ResponseBean getList(@RequestParam(defaultValue = "1") Integer pageNum,
                                @RequestParam(defaultValue = "10") Integer pageSize,
                                @RequestParam(required = false) String name,
                                @RequestParam(required = false) Date beginDate,
                                @RequestParam(required = false) Date endDate) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityService.getList(pageNum, pageSize, name, beginDate, endDate);
        return responseBean;
    }

    @RequestMapping(value = "/activity/{activityId}", method = RequestMethod.GET)
    @ApiOperation(value = "活动详情", notes = "活动详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = String.class, name = "activityId", value = "activityId活动id")
    })
    public ResponseBean getById(@PathVariable String activityId) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityService.getActivityId(activityId);
        return responseBean;
    }

    @RequestMapping(value = "/activity/add", method = RequestMethod.POST)
    @ApiOperation(value = "添加活动", notes = "添加活动")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "title活动标题;activityDate活动时间;beginDate活动开始时间;endDate活动截止时间;maxNum人数限制;context内容;imgUrl图片地址")
    })
    public ResponseBean add(@RequestBody String parameter) {
        ResponseBean responseBean = new ResponseBean();
        Activity activity = JSONObject.parseObject(parameter, Activity.class);
        activity.setStatus(0);
        activity.setActivityId(idGen.nextId()+"");
        activity.setCreateDate(LocalDateTime.now());
        iActivityService.save(activity);
        return responseBean;
    }

    @RequestMapping(value = "/activity/send/{activityId}", method = RequestMethod.PUT)
    @ApiOperation(value = "活动发布", notes = "活动发布")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = String.class, name = "activityId", value = "activityId活动编号")
    })
    public ResponseBean send(@PathVariable String activityId) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<Activity> activityQueryWrapper = new QueryWrapper<>();
        activityQueryWrapper.eq("activity_id", activityId);
        Activity activity = iActivityService.getOne(activityQueryWrapper);
        activity.setStatus(1);
        iActivityService.update(activity, activityQueryWrapper);
        return responseBean;
    }

    @RequestMapping(value = "/activity/cancel/{activityId}", method = RequestMethod.PUT)
    @ApiOperation(value = "活动取消发布", notes = "活动取消发布")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = String.class, name = "activityId", value = "activityId活动编号")
    })
    public ResponseBean cancel(@PathVariable String activityId) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<Activity> activityQueryWrapper = new QueryWrapper<>();
        activityQueryWrapper.eq("activity_id", activityId);
        Activity activity = iActivityService.getOne(activityQueryWrapper);
        activity.setStatus(0);
        iActivityService.update(activity, activityQueryWrapper);
        return responseBean;
    }

    @RequestMapping(value = "/activity/update", method = RequestMethod.PUT)
    @ApiOperation(value = "修改活动", notes = "修改活动")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "activityId活动编号;title活动标题;activityDate活动时间;beginDate活动开始时间;endDate活动截止时间;maxNum人数限制;context内容;imgUrl图片地址")
    })
    public ResponseBean update(@RequestBody String parameter) {
        ResponseBean responseBean = new ResponseBean();
        Activity activity = JSONObject.parseObject(parameter, Activity.class);
        QueryWrapper<Activity> activityQueryWrapper = new QueryWrapper<>();
        activityQueryWrapper.eq("activity_id", activity.getActivityId());
        iActivityService.update(activity, activityQueryWrapper);
        return responseBean;
    }

    @RequestMapping(value = "/activity/delete/{activityId}", method = RequestMethod.DELETE)
    @ApiOperation(value = "删除活动", notes = "删除活动")
    public ResponseBean delete(@PathVariable String activityId) {
        ResponseBean responseBean = new ResponseBean();
        QueryWrapper<Activity> activityQueryWrapper = new QueryWrapper<>();
        activityQueryWrapper.eq("activity_id", activityId);
        Activity activity = iActivityService.getOne(activityQueryWrapper);
        if (null == activity) {
            responseBean.addError("活动不存在!");
            return responseBean;
        }
        if (activity.getStatus() == 1) {
            responseBean.addError("已发布的活动不能删除!");
            return responseBean;
        }
        activity.setStatus(-1);
        iActivityService.update(activity, activityQueryWrapper);
        responseBean.addSuccess("操作成功!");
        return responseBean;
    }


    @RequestMapping(value = "/activity/enroll/list/{activityId}", method = RequestMethod.POST)
    @ApiOperation(value = "根据活动id查询活动报名列表", notes = "根据活动id查询活动报名列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageNum", value = "pageNum第几页"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageSize", value = "pageSize一页多少行"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = String.class, name = "customerName", value = "客户端用户名"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = String.class, name = "customerPhone", value = "客户端用户手机"),
    })
    public ResponseBean activityEnrollList(@PathVariable("activityId") String activityId,
                                           @RequestParam(defaultValue = "1") Integer pageNum,
                                           @RequestParam(defaultValue = "10") Integer pageSize,
                                           @RequestParam(required = false) String customerName,
                                           @RequestParam(required = false) String customerPhone) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityEnrollService.getActivitySignUpList(activityId, pageNum, pageSize, customerName, customerPhone);
        return responseBean;
    }

    @RequestMapping(value = "/wx/activity/byuser/{openid}", method = RequestMethod.GET)
    @ApiOperation(value = "微信小程序 根据openid查询参加过的活动列表", notes = "微信小程序 根据openid查询参加过的活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageNum", value = "pageNum第几页"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageSize", value = "pageSize一页多少行")
    })
    public ResponseBean getOpenidActivityUpList(@PathVariable("openid") String openid, @RequestParam(defaultValue = "1") Integer pageNum, @RequestParam(defaultValue = "10") Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityEnrollService.getOpenidActivityUpList(openid, pageNum, pageSize);
        return responseBean;
    }

    @RequestMapping(value = "/wx/signUpActivity/{openid}", method = RequestMethod.POST)
    @ApiOperation(value = "微信小程序活动报名", notes = "微信小程序活动报名")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "body", dataTypeClass = String.class, name = "parameter", value = "activityId活动编号;enrollDate报名时间"),
    })
    public ResponseBean wxSignUpActivity(@PathVariable("openid") String openid, @RequestBody String parameter) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityEnrollService.wxSignUpActivity(openid, parameter);
        return responseBean;
    }

    @RequestMapping(value = "/wx/activity", method = RequestMethod.GET)
    @ApiOperation(value = "微信活动列表", notes = "微信活动列表")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageNum", value = "pageNum第几页"),
            @ApiImplicitParam(paramType = "query", dataTypeClass = Integer.class, name = "pageSize", value = "pageSize一页多少行")
    })
    public ResponseBean getWxList(@RequestParam(defaultValue = "1") Integer pageNum,
                                @RequestParam(defaultValue = "10") Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityService.getList(pageNum, pageSize, null,null,null);
        return responseBean;
    }

    @RequestMapping(value = "/wx/activity/{activityId}", method = RequestMethod.GET)
    @ApiOperation(value = "活动详情", notes = "活动详情")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType = "path", dataTypeClass = String.class, name = "activityId", value = "activityId活动id")
    })
    public ResponseBean getWxById(@PathVariable String activityId) {
        ResponseBean responseBean = new ResponseBean();
        responseBean = iActivityService.getWxActivityId(activityId);
        return responseBean;
    }
}
